#include "cDeck.h"

#include <vector>

using namespace std;

void OutProb(cStrat s = sHITSOFT17)
{
	//output the first line
	cout << "Dealer's-up-card 17 18 19 20 21 Bust" << endl;

	//figure out the overall probility
	vector<double> overall(22, 0);

	//output the probility of ending at each possible values given dealer's up card
	cDeck deck;

	for(int i=2; i<=11; i++)
	{
		vector<double> prob;

		//figure out the probility
		if (i == 11)
		{
			prob = deck.Prob(i, true, s);
		}
		else
		{
			prob = deck.Prob(i, false, s);
		}

		//output the first number
		cout.width(2);
		cout << i << ' ';
		/*
		if (i == 10) cout << "10;J;Q;K" << endl;
		else if (i == 11) cout << "A" << endl;
		else cout << i << ' ';
		*/

		//for those below 21, i.e. 17 to 21
		double totalp = 0.0;
		cout.precision(7);
		cout.setf(ios::fixed);
		for(int r=17; r<=21; r++)
		{
			cout << prob[r] << ' ';
			totalp += prob[r];
		}

		//the probility for anything below 16 should be 0;
		assert(prob[16] == 0);

		//NOTE: for over 21, accumulate all prob to 16
		double sum = 0.0;
		for(int b=22; b<prob.size(); b++)
		{
			sum += prob[b];
			totalp += prob[b];
			prob[16] += prob[b];
		}

		//probility should add up to 1
		assert(fabs(totalp - 1.0) < 0.001);

		cout << sum << endl;

		//add the probility to overall
		for(int o=16; o<=21; o++)
		{
			//there are 4 cards that represent 10, take that into account
			double mult;
			if (i == 10) mult = 4;
			else		mult = 1;

			//there are 13 different possible cards
			mult /= 13.0;

			overall[o] += prob[o] * mult;
		}

	}

	//for checking
	double allp = 0;

	//output overall results
	cout << "Overall ";
	for(int out=17; out<=21; out++)
	{
		allp += overall[out];
		cout << overall[out] << ' ';
	}

	//output the probility for bust
	allp += overall[16];
	cout << overall[16] << endl;

	assert(fabs(allp - 1) < 0.0001);

	return;
}


int main(void)
{
	OutProb(sHIT16);
	OutProb(sHITSOFT17);
	return 0;
}
