#include "cDeck.h"


cDeck::cDeck(int deck, bool ten)
{
	bTen = ten;
	nDeck = deck;
	Reset();
}

void cDeck::Reset()
{
	HighLowIndex = 0;
	Card.erase(Card.begin(), Card.end());
	Deck.erase(Deck.begin(), Deck.end());

	Total = 52 * nDeck;

	Card.resize(13);

	for(int i=0; i<13; i++)
	{
		Card[i] = 4 * nDeck;
	}

	for(int j=0; j<13; j++)
	{
		for(int k=0; k<4*nDeck; k++)
		{
			Deck.push_back(j);
		}
	}

	random_shuffle(Deck.begin(), Deck.end());
}

int cDeck::NumLeft(int n)
{
	if (!bTen)
	{
		return Card[n-1];
	}
	else
	{
		if (n >= 1 && n <= 9)
		{
			return Card[n-1];
		}
		else if (n == 11)
		{
			return NumLeft(1);
		}
		else if (n == 12 || n == 13)
		{
			return 0;
		}
		else if (n == 10)
		{
			return	Card[10-1] +
					Card[11-1] +
					Card[12-1] +
					Card[13-1];
		}
		else
		{
			return 0;
		}
	}
}

int cDeck::Deal()
{
	if (Total == 0)
	{
		
		cout << "No more card, shuffle then deal" << endl;
		Reset();
		return Deal();
	}

	int Top = Deck.back();
	Deck.pop_back();

	Card[Top]--;
	Total--;

	int card = Top + 1;
	
	if (card >= 10 && card <= 13 || card == 1) HighLowIndex--;
	else if (card >= 2 && card <= 6) HighLowIndex++;

	if (bTen)
	{
		if (card == 11 || card == 12 || card == 13) return 10;
	}
	
	return card;
}

int cDeck::ForceDeal(cCard card)
{
	if (NumLeft() == 0)
	{
		cout << "No more card!!" << endl;
		Reset();
		return ForceDeal(card);
	}

	if (NumLeft(card) == 0 && !bTen) //remove this check when bTen is activiated
	{
		cout << "The card to be forced does not exist, another card is dealt" << endl;
		return Deal();
	}

	int Index = card - 1;
	
	for(int d=0; d<Deck.size(); d++)
	{
		if (Deck[d] == Index)
		{
			swap(Deck[d], Deck[Deck.size()-1]);
			return Deal();
		}
	}

	//it should never get here
	assert(false);
}

void cDeck::Output()
{
	cout << "Number of Card: " << NumLeft() << endl;
	
	int Max;
	
	if (bTen) Max = 10;
	else Max = 13;

	for(int i=0; i<Max; i++)
	{
		cout << "\t" << "Num of " << i+1 << " is " << NumLeft(i+1) << endl;
	}

	cout << endl;
}

double cDeck::HighLowCount(void)
{
	assert(-HighLowIndex ==	Card[1] + Card[2] + Card[3] + Card[4] + Card[5] -
							Card[0] - Card[9] - Card[10]- Card[11]- Card[12]);

	if (NumLeft() == 0) return 0.0;
	return double(HighLowIndex)  / (double(NumLeft()) / 52.0);
}

//Return a vector of 1 to SOMETHINGBIG, containing the probility of getting each
vector<double> cDeck::Prob(const int nTotal, bool bSoft, cStrat s)
{
	int STANDAT; //when should comp stop at hard total, not sure what happen if this is set below 11
	int SOFTSTANDAT;	//when should comp stop at soft total
	int MAX;		//maximun number of points, not sure what happen if this is greater than 22, (two ace might be problematic)
	bool ACEIS11;	//should ACE always be counted as 11
	bool FIRSTACEIS11; //should first ace be counted as 11, the rest are 1
	
	if (s == sHITSOFT17)
	{
		STANDAT = 17;
		SOFTSTANDAT = 18;
		MAX = 21;
		ACEIS11 = false;
		FIRSTACEIS11 = false;
	}
	else if (s == sHIT16)
	{
		STANDAT = 17;
		SOFTSTANDAT = 17;
		MAX = 21;
		ACEIS11 = false;
		FIRSTACEIS11 = false;
	}
	else assert(false);

	
	if (ACEIS11 && FIRSTACEIS11)
	{
		cout << "Only one of the above can be true" << endl;
		ACEIS11 = false;
	}

	//when first ace is 11, soft total is those with one ace

	vector<double> ThisSoft(MAX + 11, 0);
	vector<double> ThisHard(MAX + 11, 0);

	//when ace counts as 11, there is no soft total
	if (ACEIS11) bSoft = false;
	
	if (bSoft)
	{
		ThisSoft[nTotal] = 1.0;
	}
	else
	{
		ThisHard[nTotal] = 1.0;
	}

	for(;;)
	{
		//break when no probility for everything below STANDAT
		bool end = true;
		for(int s=0; s<STANDAT; s++)
		{
			if (ThisHard[s] != 0)
			{
				end = false;
				break;
			}
		}

		for(s=0; s<SOFTSTANDAT; s++)
		{
			if (ThisSoft[s] != 0)
			{
				end = false;
				break;
			}
		}

		if (end) break;

		//figure out the result of next round
		// i.e. deal another card or stand
		vector<double> NextHard(MAX + 11, 0);
		vector<double> NextSoft(MAX + 11, 0);
		
		//for the case of hit for hard total
		for(int current = 0; current < STANDAT; current++)
		{
			if (ThisHard[current] == 0) continue;

			for(int nextcard = 1; nextcard <= 10; nextcard++)
			{
				double prob = ThisHard[current] * (double(NumLeft(nextcard)) / double(NumLeft()));
				if (nextcard == 1)
				{
					if (ACEIS11)
					{
						NextHard[current + 11] += prob;
					}
					else if (FIRSTACEIS11)
					{
						NextSoft[current + 11] += prob;
					}
					else
					{
						if (current <= 10)
						{
							NextSoft[current + 11] += prob;
						}
						else
						{
							NextHard[current + 1] += prob;
						}
					}
				}
				else
				{
					NextHard[current + nextcard] += prob;
				}
			}
		}

		//for soft total
		for(current = 0; current < SOFTSTANDAT; current++)
		{
			if (ThisSoft[current] == 0) continue;
			for(int nextcard = 1; nextcard <= 10; nextcard++)
			{
				double prob = ThisSoft[current] * (double(NumLeft(nextcard)) / double(NumLeft()));
				
				//if blow up, then it's no longer soft total
				if (!FIRSTACEIS11 && current + nextcard > MAX)
				{
					NextHard[current + nextcard - 10] += prob;
				}
				else
				{	//no need to handle ace differently, because it will be counted 1
					NextSoft[current + nextcard] += prob;
				}
			}
		}

		//for the case of stand
		for(current = STANDAT; current < ThisHard.size(); current++)
		{
			NextHard[current] += ThisHard[current];
		}

		for(current = SOFTSTANDAT; current < ThisSoft.size(); current++)
		{
			NextSoft[current] += ThisSoft[current];
		}

		ThisSoft = NextSoft;
		ThisHard = NextHard;

		//error check
		assert(ThisSoft.size() == ThisHard.size());
		double totalp = 0;
		for(int i=0 ;i<ThisSoft.size(); i++)
		{
			totalp += ThisSoft[i] + ThisHard[i];
		}
		assert(fabs(totalp - 1.0) <= 0.01);
	}

	vector<double> Res(ThisHard.size());

	for(int i=0; i<ThisHard.size(); i++)
	{
		Res[i] = ThisHard[i] + ThisSoft[i];
	}

	return Res;
}
